@@ -9,6 +9,7 @@ import shutil
 
 # Import salt testing libs
 from tests.support.case import SSHCase
+from tests.support.runtests import RUNTIME_VARS
 
 
 class SSHTest(SSHCase):
@@ -39,3 +40,12 @@ class SSHTest(SSHCase):
         salt_dir = self.run_function('config.get', ['thin_dir'], wipe=False)
         if os.path.exists(salt_dir):
             shutil.rmtree(salt_dir)
+
+    def test_tty(self):
+        """
+        test using tty
+        """
+        roster = os.path.join(RUNTIME_VARS.TMP, "roster-tty")
+        self.custom_roster(roster, data={"tty": True})
+        ret = self.run_function("test.ping", roster_file=roster)
+        assert ret is True
